@@ -1,34 +1,13 @@
-// Copyright 2017 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2017 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 // {{PREAMBLE_ADDITIONS}}
 
 var STACK_ALIGN = {{{ STACK_ALIGN }}};
 
-#if ASSERTIONS
-// stack management, and other functionality that is provided by the compiled code,
-// should not be used before it is ready
-
-#if DECLARE_ASM_MODULE_EXPORTS // These functions should not be defined with 'var' if DECLARE_ASM_MODULE_EXPORTS==0 (or the assignments won't be seen)
-/** @suppress{duplicate} */
-var stackSave;
-/** @suppress{duplicate} */
-var stackRestore;
-/** @suppress{duplicate} */
-var stackAlloc;
-#endif
-
-stackSave = stackRestore = stackAlloc = function() {
-  abort('cannot use the stack before compiled code is ready to run, and has provided stack access');
-};
-
-function staticAlloc(size) {
-  abort('staticAlloc is no longer available at runtime; instead, perform static allocations at compile time (using makeStaticAlloc)');
-}
-#endif
-
 function dynamicAlloc(size) {
 #if ASSERTIONS
   assert(DYNAMICTOP_PTR);
@@ -124,6 +103,15 @@ function fetchBinary(url) {
   });
 }
 
+function asmjsMangle(x) {
+#if WASM_BACKEND
+  var unmangledSymbols = {{{ buildStringArray(WASM_FUNCTIONS_THAT_ARE_NOT_NAME_MANGLED) }}};
+  return x.indexOf('dynCall_') == 0 || unmangledSymbols.indexOf(x) != -1 ? x : '_' + x;
+#else
+  return x;
+#endif
+}
+
 // loadDynamicLibrary loads dynamic library @ lib URL / path and returns handle for loaded DSO.
 //
 // Several flags affect the loading:
@@ -271,10 +259,8 @@ function loadDynamicLibrary(lib, flags) {
       //
       // We should copy the symbols (which include methods and variables) from SIDE_MODULE to MAIN_MODULE.
 
-      var module_sym = sym;
-#if WASM_BACKEND
-      module_sym = '_' + sym;
-#else
+      var module_sym = asmjsMangle(sym);
+#if !WASM_BACKEND
       // Module of SIDE_MODULE has not only the symbols (which should be copied)
       // but also others (print*, asmGlobal*, FUNCTION_TABLE_**, NAMED_GLOBALS, and so on).
       //
@@ -282,7 +268,7 @@ function loadDynamicLibrary(lib, flags) {
       // When the symbol (which should be copied) is variable, Module.* 's type becomes number.
       // Except for the symbol prefix (_), there is no difference in the symbols (which should be copied) and others.
       // So this just copies over compiled symbols (which start with _).
-      if (sym[0] !== '_') {
+      if (!sym.startsWith('dynCall_') && sym[0] !== '_') {
         continue;
       }
 #endif
@@ -338,10 +324,10 @@ function relocateExports(exports, memoryBase, tableBase, moduleLocal) {
 #if EMULATE_FUNCTION_POINTER_CASTS
       // it may be a function pointer
       if (e.substr(0, 3) == 'fp$' && typeof exports[e.substr(3)] === 'function') {
-        value = value + tableBase;
+        value += tableBase;
       } else {
 #endif
-        value = value + memoryBase;
+        value += memoryBase;
 #if EMULATE_FUNCTION_POINTER_CASTS
       }
 #endif
@@ -358,6 +344,24 @@ function relocateExports(exports, memoryBase, tableBase, moduleLocal) {
   return relocated;
 }
 
+#if RELOCATABLE
+// Dynmamic version of shared.py:make_invoke.  This is needed for invokes
+// that originate from side modules since these are not known at JS
+// generation time.
+function createInvokeFunction(sig) {
+  return function() {
+    var sp = stackSave();
+    try {
+      return Module['dynCall_' + sig].apply(null, arguments);
+    } catch(e) {
+      stackRestore(sp);
+      if (e !== e+0 && e !== 'longjmp') throw e;
+      _setThrew(1, 0);
+    }
+  }
+}
+#endif
+
 // Loads a side module from binary data
 function loadWebAssemblyModule(binary, flags) {
   var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
@@ -415,12 +419,6 @@ function loadWebAssemblyModule(binary, flags) {
 #endif
     // prepare memory
     var memoryBase = alignMemory(getMemory(memorySize + memoryAlign), memoryAlign); // TODO: add to cleanups
-    // The static area consists of explicitly initialized data, followed by zero-initialized data.
-    // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
-    // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
-    // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
-    // from uninitialized memory.
-    for (var i = memoryBase; i < memoryBase + memorySize; ++i) HEAP8[i] = 0;
     // prepare env imports
     var env = asmLibraryArg;
     // TODO: use only __memory_base and __table_base, need to update asm.js backend
@@ -430,7 +428,11 @@ function loadWebAssemblyModule(binary, flags) {
     table.grow(tableSize);
     assert(table === originalTable);
     // zero-initialize memory and table
-    // TODO: in some cases we can tell it is already zero initialized
+    // The static area consists of explicitly initialized data, followed by zero-initialized data.
+    // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
+    // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
+    // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
+    // from uninitialized memory.
     for (var i = memoryBase; i < memoryBase + memorySize; i++) {
       HEAP8[i] = 0;
     }
@@ -445,23 +447,30 @@ function loadWebAssemblyModule(binary, flags) {
     var moduleLocal = {};
 
     var resolveSymbol = function(sym, type, legalized) {
+#if WASM_BIGINT
+      assert(!legalized);
+#else
       if (legalized) {
         sym = 'orig$' + sym;
       }
+#endif
 
       var resolved = Module["asm"][sym];
       if (!resolved) {
-#if WASM_BACKEND
-        sym = '_' + sym;
-#endif
-        resolved = Module[sym];
+        var mangled = asmjsMangle(sym);
+        resolved = Module[mangled];
         if (!resolved) {
-          resolved = moduleLocal[sym];
+          resolved = moduleLocal[mangled];
         }
+#if RELOCATABLE
+        if (!resolved && sym.startsWith('invoke_')) {
+          resolved = createInvokeFunction(sym.split('_')[1]);
+        }
+#endif
 #if ASSERTIONS
         assert(resolved, 'missing linked ' + type + ' `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
 #endif
-     }
+      }
       return resolved;
     }
 
@@ -511,7 +520,11 @@ function loadWebAssemblyModule(binary, flags) {
           assert(parts.length == 3)
           var name = parts[1];
           var sig = parts[2];
+#if WASM_BIGINT
+          var legalized = false;
+#else
           var legalized = sig.indexOf('j') >= 0; // check for i64s
+#endif
           var fp = 0;
           return obj[prop] = function() {
             if (!fp) {
@@ -521,13 +534,6 @@ function loadWebAssemblyModule(binary, flags) {
             return fp;
           };
         }
-        if (prop.startsWith('invoke_')) {
-          // A missing invoke, i.e., an invoke for a function type
-          // present in the dynamic library but not in the main JS,
-          // and the dynamic library cannot provide JS for it. Use
-          // the generic "X" invoke for it.
-          return obj[prop] = invoke_X;
-        }
         // otherwise this is regular function import - call it indirectly
         return obj[prop] = function() {
           return resolveSymbol(prop, 'function').apply(null, arguments);
@@ -754,15 +760,6 @@ function getCompilerSetting(name) {
 #endif // ASSERTIONS
 #endif // RETAIN_COMPILER_SETTINGS
 
-var Runtime = {
-#if ASSERTIONS
-  // helpful errors
-  getTempRet0: function() { abort('getTempRet0() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-  staticAlloc: function() { abort('staticAlloc() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-  stackAlloc: function() { abort('stackAlloc() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-#endif
-};
-
 // The address globals begin at. Very low in memory, for code size and optimization opportunities.
 // Above 0 is static memory, starting with globals.
 // Then the stack.
@@ -780,4 +777,3 @@ var Atomics_load = Atomics.load;
 var Atomics_store = Atomics.store;
 var Atomics_compareExchange = Atomics.compareExchange;
 #endif
-
