@@ -25,19 +25,9 @@ long* _get_timezone() {
   return &timezone;
 }
 
-char** environ;
+// Musl lock internals. As we assume wasi is single-threaded for now, these
+// are no-ops.
 
-char*** _get_environ() {
-  return &environ;
-}
-
-// Call JS to build the default environment.
-
-extern void __buildEnvironment(void*);
-
-// TODO: this needs very high priority, so user ctors that use environ do not happen first
-__attribute__((constructor))
-void __emscripten_environ_constructor(void) {
-  __buildEnvironment((void*)&environ);
-}
+void __lock(void* ptr) {}
+void __unlock(void* ptr) {}
 
