@@ -1,7 +1,6 @@
 var acorn = require('acorn');
-var terser = require("terser");
+var terser = require('../third_party/terser');
 var fs = require('fs');
-var path = require('path');
 
 // Setup
 
@@ -88,7 +87,6 @@ function fullWalk(node, c) {
 // Recursive post-order walk, calling properties on an object by node type,
 // if the type exists, and if so leaving recursion to that function.
 function recursiveWalk(node, cs) {
-//print('recw1 ' + JSON.stringify(node));
   (function c(node) {
     if (!(node.type in cs)) {
       visitChildren(node, function(child) {
@@ -125,6 +123,23 @@ function convertToNull(node) {
   node.name = 'null';
 }
 
+function convertToNullStatement(node) {
+  node.type = 'ExpressionStatement';
+  node.expression = {
+    type: "Literal",
+    value: null,
+    raw: "null",
+    start: 0,
+    end: 0,
+  };
+  node.start = 0;
+  node.end = 0;
+}
+
+function isNull(node) {
+  return node.type === 'Literal' && node.raw === 'null';
+}
+
 function setLiteralValue(item, value) {
   item.value = value;
   item.raw = "'" + value + "'";
@@ -206,7 +221,8 @@ function hasSideEffects(node) {
       case 'Literal':
       case 'Identifier':
       case 'UnaryExpression':
-      case 'BinaryExpresson':
+      case 'BinaryExpression':
+      case 'LogicalExpression':
       case 'ExpressionStatement':
       case 'UpdateOperator':
       case 'ConditionalExpression':
@@ -231,9 +247,26 @@ function hasSideEffects(node) {
         }
         break;
       }
+      case 'NewExpression': {
+        // default to unsafe, but can be safe on some familiar objects
+        if (node.callee.type === 'Identifier') {
+          var name = node.callee.name;
+          if (name === 'TextDecoder'  || name === 'ArrayBuffer' ||
+              name === 'Int8Array'    || name === 'Uint8Array' ||
+              name === 'Int16Array'   || name === 'Uint16Array' ||
+              name === 'Int32Array'   || name === 'Uint32Array' ||
+              name === 'Float32Array' || name === 'Float64Array') {
+            // no side effects, but the arguments might (we walk them in
+            // full walk as well)
+            break;
+          }
+        }
+        // not one of the safe cases
+        has = true;
+        break;
+      }
       default: {
         has = true;
-        //console.error('because ' + node.type);
       }
     }
   });
@@ -254,7 +287,7 @@ function hasSideEffects(node) {
 // as they appear (like ArrowFunctionExpression). Instead, we do a conservative
 // analysis here.
 
-function JSDCE(ast, multipleIterations) {
+function JSDCE(ast, aggressive) {
   function iteration() {
     var removed = 0;
     var scopes = [{}]; // begin with empty toplevel scope
@@ -293,6 +326,14 @@ function JSDCE(ast, multipleIterations) {
             node.oldDeclarations = old;
           }
         },
+        ExpressionStatement(node, c) {
+          if (aggressive && !hasSideEffects(node)) {
+            if (!isNull(node.expression)) {
+              convertToNullStatement(node);
+              removed++;
+            }
+          }
+        },
         FunctionDeclaration(node, c) {
           if (Object.prototype.hasOwnProperty.call(names, node.id.name)) {
             removed++;
@@ -354,6 +395,14 @@ function JSDCE(ast, multipleIterations) {
           c(node.value);
         });
       },
+      MemberExpression(node, c) {
+        c(node.object);
+        // Ignore a property identifier (a.X), but notice a[X] (computed
+        // is true) and a["X"] (it will be a Literal and not Identifier).
+        if (node.property.type !== 'Identifier' || node.computed) {
+          c(node.property);
+        }
+      },
       FunctionDeclaration(node, c) {
         handleFunction(node, c, true /* defun */);
       },
@@ -385,12 +434,12 @@ function JSDCE(ast, multipleIterations) {
     cleanUp(ast, names);
     return removed;
   }
-  while (iteration() && multipleIterations) { }
+  while (iteration() && aggressive) { }
 }
 
 // Aggressive JSDCE - multiple iterations
 function AJSDCE(ast) {
-  JSDCE(ast, /* multipleIterations= */ true);
+  JSDCE(ast, /* aggressive= */ true);
 }
 
 function isAsmLibraryArgAssign(node) { // var asmLibraryArg = ..
@@ -685,7 +734,7 @@ function emitDCEGraph(ast) {
               value = assign.right;
             }
           }
-          if (target.type === 'Identifier' && target.name === 'asm' && value) {
+          if (target && target.type === 'Identifier' && target.name === 'asm' && value) {
             if (value.type === 'MemberExpression' &&
                 value.object.type === 'MemberExpression' &&
                 value.object.object.type === 'Identifier' &&
@@ -903,6 +952,20 @@ function makeCallExpression(node, name, args) {
   });
 }
 
+function isEmscriptenHEAP(name) {
+  switch (name) {
+    case 'HEAP8':  case 'HEAPU8':
+    case 'HEAP16': case 'HEAPU16':
+    case 'HEAP32': case 'HEAPU32':
+    case 'HEAPF32': case 'HEAPF64': {
+      return true;
+    }
+    default: {
+      return false;
+    }
+  }
+}
+
 // Instrument heap accesses to call GROWABLE_HEAP_* helper functions instead, which allows
 // pthreads + memory growth to work (we check if the memory was grown on another thread
 // in each access), see #8365.
@@ -910,19 +973,9 @@ function growableHeap(ast) {
   recursiveWalk(ast, {
     AssignmentExpression: function(node) {
       if (node.left.type === 'Identifier' &&
-          node.left.name.startsWith('HEAP')
-      ) {
+          isEmscriptenHEAP(node.left.name)) {
         // Don't transform initial setup of the arrays.
-        switch (node.left.name) {
-          case 'HEAP8':  case 'HEAPU8':
-          case 'HEAP16': case 'HEAPU16':
-          case 'HEAP32': case 'HEAPU32':
-          case 'HEAPF32': case 'HEAPF64': {
-            return;
-          }
-          default: {
-          }
-        }
+        return;
       }
       growableHeap(node.left);
       growableHeap(node.right);
@@ -980,6 +1033,193 @@ function growableHeap(ast) {
   });
 }
 
+// Make all JS pointers unsigned. We do this by modifying things like
+// HEAP32[X >> 2] to HEAP32[X >>> 2]. We also need to handle the case of
+// HEAP32[X] and make that HEAP32[X >>> 0], things like subarray(), etc.
+function unsignPointers(ast) {
+  // Aside from the standard emscripten HEAP*s, also identify just "HEAP"/"heap"
+  // as representing a heap. This can be used in JS library code in order
+  // to get this pass to fix it up.
+  function isHeap(name) {
+    return isEmscriptenHEAP(name) || name === 'heap' || name === 'HEAP';
+  }
+
+  function unsign(node) {
+    // The pointer is often a >> shift, which we can just turn into >>>
+    if (node.type === 'BinaryExpression') {
+      if (node.operator === '>>') {
+        node.operator = '>>>';
+        return node;
+      }
+    }
+    // If nothing else worked out, add a new shift.
+    return {
+      type: 'BinaryExpression',
+      left: node,
+      operator: ">>>",
+      right: {
+        type: "Literal",
+        value: 0,
+        raw: "0",
+        start: 0,
+        end: 0,
+      },
+      start: 0,
+      end: 0,
+    };
+  }
+
+  fullWalk(ast, function(node) {
+    if (node.type === 'MemberExpression') {
+      // Check if this is HEAP*[?]
+      if (node.object.type === 'Identifier' &&
+          isHeap(node.object.name) &&
+          node.computed) {
+        node.property = unsign(node.property);
+      }
+    } else if (node.type === 'CallExpression') {
+      if (node.callee.type === 'MemberExpression' &&
+          node.callee.object.type === 'Identifier' &&
+          isHeap(node.callee.object.name) &&
+          node.callee.property.type === 'Identifier' &&
+          !node.computed) {
+        // This is a call on HEAP*.?. Specific things we need to fix up are
+        // subarray, set, and copyWithin. TODO more?
+        if (node.callee.property.name === 'set') {
+          if (node.arguments.length >= 2) {
+            node.arguments[1] = unsign(node.arguments[1]);
+          }
+        } else if (node.callee.property.name === 'subarray') {
+          if (node.arguments.length >= 1) {
+            node.arguments[0] = unsign(node.arguments[0]);
+            if (node.arguments.length >= 2) {
+              node.arguments[1] = unsign(node.arguments[1]);
+            }
+          }
+        } else if (node.callee.property.name === 'copyWithin') {
+          node.arguments[0] = unsign(node.arguments[0]);
+          node.arguments[1] = unsign(node.arguments[1]);
+          if (node.arguments.length >= 3) {
+            node.arguments[2] = unsign(node.arguments[2]);
+          }
+        }
+      }
+    }
+  });
+}
+
+// Replace direct HEAP* loads/stores with calls into C, in which ASan checks
+// are applied. That lets ASan cover JS too.
+function asanify(ast) {
+  function isHEAPAccess(node) {
+    return node.type === 'MemberExpression' &&
+           node.object.type === 'Identifier' &&
+           node.computed && // notice a[X] but not a.X
+           isEmscriptenHEAP(node.object.name);
+  }
+
+  recursiveWalk(ast, {
+    FunctionDeclaration(node, c) {
+      if (node.id.type === 'Identifier' && node.id.name.startsWith('_asan_js_')) {
+        // do not recurse into this js impl function, which we use during
+        // startup before the wasm is ready
+      } else {
+        c(node.body);
+      }
+    },
+    AssignmentExpression(node, c) {
+      var target = node.left;
+      var value = node.right;
+      c(value);
+      if (isHEAPAccess(target)) {
+        // Instrument a store.
+        var ptr = target.property;
+        switch (target.object.name) {
+          case 'HEAP8': {
+            makeCallExpression(node, '_asan_js_store_1', [ptr, value]);
+            break;
+          }
+          case 'HEAPU8': {
+            makeCallExpression(node, '_asan_js_store_1u', [ptr, value]);
+            break;
+          }
+          case 'HEAP16': {
+            makeCallExpression(node, '_asan_js_store_2', [ptr, value]);
+            break;
+          }
+          case 'HEAPU16': {
+            makeCallExpression(node, '_asan_js_store_2u', [ptr, value]);
+            break;
+          }
+          case 'HEAP32': {
+            makeCallExpression(node, '_asan_js_store_4', [ptr, value]);
+            break;
+          }
+          case 'HEAPU32': {
+            makeCallExpression(node, '_asan_js_store_4u', [ptr, value]);
+            break;
+          }
+          case 'HEAPF32': {
+            makeCallExpression(node, '_asan_js_store_f', [ptr, value]);
+            break;
+          }
+          case 'HEAPF64': {
+            makeCallExpression(node, '_asan_js_store_d', [ptr, value]);
+            break;
+          }
+          default: {}
+        }
+      } else {
+        c(target);
+      }
+    },
+    MemberExpression(node, c) {
+      c(node.property);
+      if (!isHEAPAccess(node)) {
+        c(node.object);
+      } else {
+        // Instrument a load.
+        var ptr = node.property;
+        switch (node.object.name) {
+          case 'HEAP8': {
+            makeCallExpression(node, '_asan_js_load_1', [ptr]);
+            break;
+          }
+          case 'HEAPU8': {
+            makeCallExpression(node, '_asan_js_load_1u', [ptr]);
+            break;
+          }
+          case 'HEAP16': {
+            makeCallExpression(node, '_asan_js_load_2', [ptr]);
+            break;
+          }
+          case 'HEAPU16': {
+            makeCallExpression(node, '_asan_js_load_2u', [ptr]);
+            break;
+          }
+          case 'HEAP32': {
+            makeCallExpression(node, '_asan_js_load_4', [ptr]);
+            break;
+          }
+          case 'HEAPU32': {
+            makeCallExpression(node, '_asan_js_load_4u', [ptr]);
+            break;
+          }
+          case 'HEAPF32': {
+            makeCallExpression(node, '_asan_js_load_f', [ptr]);
+            break;
+          }
+          case 'HEAPF64': {
+            makeCallExpression(node, '_asan_js_load_d', [ptr]);
+            break;
+          }
+          default: {}
+        }
+      }
+    }
+  });
+}
+
 function reattachComments(ast, comments) {
   var symbols = [];
 
@@ -998,10 +1238,12 @@ function reattachComments(ast, comments) {
   // Walk through all comments in ascending line number, and match each
   // comment to the appropriate code block.
   for(var i = 0, j = 0; i < comments.length; ++i) {
-    while(j < symbols.length && symbols[j].start.pos < comments[i].end)
+    while(j < symbols.length && symbols[j].start.pos < comments[i].end) {
       ++j;
-    if (j >= symbols.length)
+    }
+    if (j >= symbols.length) {
       break;
+    }
     if (symbols[j].start.pos - comments[i].end > 20) {
       // This comment is too far away to refer to the given symbol. Drop
       // the comment altogether.
@@ -1056,7 +1298,25 @@ if (extraInfoStart > 0) {
 // Collect all JS code comments to this array so that we can retain them in the outputted code
 // if --closureFriendly was requested.
 var sourceComments = [];
-var ast = acorn.parse(input, { ecmaVersion: 6, preserveParens: closureFriendly, onComment: closureFriendly ? sourceComments : undefined });
+var ast;
+try {
+  ast = acorn.parse(input, {
+    ecmaVersion: 2018,
+    preserveParens: closureFriendly,
+    onComment: closureFriendly ? sourceComments : undefined
+  });
+} catch (err) {
+  err.message += (function() {
+    var errorMessage = '\n' + input.split(acorn.lineBreak)[err.loc.line - 1] + '\n';
+    var column = err.loc.column;
+    while (column--) {
+      errorMessage += ' ';
+    }
+    errorMessage += '^\n';
+    return errorMessage;
+  })();
+  throw err;
+}
 
 var minifyWhitespace = false;
 var noPrint = false;
@@ -1071,6 +1331,8 @@ var registry = {
   noPrint: function() { noPrint = true },
   dump: function() { dump(ast) },
   growableHeap: growableHeap,
+  unsignPointers: unsignPointers,
+  asanify: asanify
 };
 
 passes.forEach(function(pass) {
@@ -1092,4 +1354,3 @@ if (!noPrint) {
   });
   print(output);
 }
-
