@@ -1,26 +1,40 @@
+# Copyright 2016 The Emscripten Authors.  All rights reserved.
+# Emscripten is available under two separate licenses, the MIT license and the
+# University of Illinois/NCSA Open Source License.  Both these licenses can be
+# found in the LICENSE file.
 
 from __future__ import print_function
-import os, sys, subprocess, multiprocessing, re, string, json, shutil, logging, traceback
+import os
+import sys
+import subprocess
+import re
+import json
+import shutil
+import tempfile
+import logging
+import traceback
 
 sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 
-from tools import shared
-from tools.js_optimizer import *
+from tools import shared, building
+from tools.js_optimizer import DEBUG, temp_files, start_funcs_marker, end_funcs_marker, split_funcs, start_asm_marker, end_asm_marker
+from tools.js_optimizer import MIN_CHUNK_SIZE, MAX_CHUNK_SIZE, NUM_CHUNKS_PER_CORE
+
+DUPLICATE_FUNCTION_ELIMINATOR = shared.path_from_root('tools', 'eliminate-duplicate-functions.js')
 
-DUPLICATE_FUNCTION_ELIMINATOR = path_from_root('tools', 'eliminate-duplicate-functions.js')
 
 def process_shell(js_engine, shell, equivalentfn_hash_info=None):
   suffix = '.eliminatedupes'
 
   with temp_files.get_file(suffix + '.js') as temp_file:
-    f = open(temp_file, 'w')
-    f.write(shell)
-    f.write('\n')
+    with open(temp_file, 'w') as f:
+      f.write(shell)
+      f.write('\n')
 
-    f.write(equivalentfn_hash_info)
-    f.close()
+      f.write(equivalentfn_hash_info)
 
-    proc = shared.run_process(js_engine +
+    proc = shared.run_process(
+        js_engine +
         [DUPLICATE_FUNCTION_ELIMINATOR, temp_file, '--use-hash-info', '--no-minimize-whitespace'],
         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
   assert len(proc.stdout)
@@ -28,6 +42,7 @@ def process_shell(js_engine, shell, equivalentfn_hash_info=None):
 
   return proc.stdout
 
+
 def run_on_chunk(command):
   try:
     file_suffix = '.js'
@@ -39,11 +54,13 @@ def run_on_chunk(command):
 
     if os.environ.get('EMCC_SAVE_OPT_TEMP') and os.environ.get('EMCC_SAVE_OPT_TEMP') != '0':
       saved = 'save_' + os.path.basename(filename)
-      while os.path.exists(saved): saved = 'input' + str(int(saved.replace('input', '').replace('.txt', ''))+1) + '.txt'
+      while os.path.exists(saved):
+        saved = 'input' + str(int(saved.replace('input', '').replace('.txt', '')) + 1) + '.txt'
       print('running DFE command', ' '.join([c if c != filename else saved for c in command]), file=sys.stderr)
       shutil.copyfile(filename, os.path.join(shared.get_emscripten_temp_dir(), saved))
 
-    if shared.EM_BUILD_VERBOSE >= 3: print('run_on_chunk: ' + str(command), file=sys.stderr)
+    if shared.EM_BUILD_VERBOSE >= 3:
+      print('run_on_chunk: ' + str(command), file=sys.stderr)
 
     proc = shared.run_process(command, stdout=subprocess.PIPE)
     output = proc.stdout
@@ -51,15 +68,15 @@ def run_on_chunk(command):
     assert len(output) and not output.startswith('Assertion failed'), 'Error in optimizer: ' + output
     filename = temp_files.get(os.path.basename(filename) + '.dfjo' + file_suffix).name
 
-    f = open(filename, 'w')
-    f.write(output)
-    f.close()
-    if DEBUG and not shared.WINDOWS: print('.', file=sys.stderr) # Skip debug progress indicator on Windows, since it doesn't buffer well with multiple threads printing to console.
+    with open(filename, 'w') as f:
+      f.write(output)
+    if DEBUG and not shared.WINDOWS:
+      print('.', file=sys.stderr) # Skip debug progress indicator on Windows, since it doesn't buffer well with multiple threads printing to console.
     return filename
   except KeyboardInterrupt:
     # avoid throwing keyboard interrupts from a child process
     raise Exception()
-  except (TypeError, ValueError) as e:
+  except (TypeError, ValueError):
     formatted_lines = traceback.format_exc().splitlines()
 
     print(">>>>>>>>>>>>>>>>>", file=sys.stderr)
@@ -69,6 +86,7 @@ def run_on_chunk(command):
 
     raise
 
+
 def dump_equivalent_functions(passed_in_filename, global_data):
   # Represents the sets of equivalent functions for the passed in filename
   equivalent_fn_info = {}
@@ -98,6 +116,7 @@ def dump_equivalent_functions(passed_in_filename, global_data):
   with open(equivalent_fn_json_file, 'w') as fout:
     fout.write(json.dumps(equivalent_fn_info))
 
+
 def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
   # Represents the aggregated info for all the json files passed in
   # Each json file contains info for one of the processed chunks
@@ -150,7 +169,8 @@ def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
     dump_equivalent_functions(passed_in_filename, global_data)
 
   # Now write the equivalent function hash to the last line of the file
-  f.write('// ' + json.dumps(equivalent_fn_hash, separators=(',',':')))
+  f.write('// ' + json.dumps(equivalent_fn_hash, separators=(',', ':')))
+
 
 # gen_hash_info is used to determine whether we are generating
 # the global set of function implementation hashes. If set to
@@ -159,7 +179,7 @@ def write_equivalent_fn_hash_to_file(f, json_files, passed_in_filename):
 # Returns the filename of the processed JS file, which is expected to be
 # deleted by the caller once done.
 def run_on_js(filename, gen_hash_info=False):
-  js_engine=shared.NODE_JS
+  js_engine = shared.NODE_JS
 
   js = open(filename).read()
   if os.linesep != '\n':
@@ -197,7 +217,7 @@ def run_on_js(filename, gen_hash_info=False):
     # we assume there is a maximum of one new name per line
     asm_shell_pre, asm_shell_post = process_shell(js_engine, asm_shell, equivalentfn_hash_info).split('EMSCRIPTEN_FUNCS();')
     asm_shell_pre = re.sub(r'(\S+\s*=\s*)ZERO\$DOT\$ZERO', r'\g<1>0.0', asm_shell_pre)
-    asm_shell_post = asm_shell_post.replace('});', '})');
+    asm_shell_post = asm_shell_post.replace('});', '})')
     pre += asm_shell_pre + '\n' + start_funcs_marker
     post = end_funcs_marker + asm_shell_post + post
 
@@ -217,26 +237,25 @@ def run_on_js(filename, gen_hash_info=False):
 
   # if we are making source maps, we want our debug numbering to start from the
   # top of the file, so avoid breaking the JS into chunks
-  cores = shared.Building.get_num_cores()
+  cores = building.get_num_cores()
 
   intended_num_chunks = int(round(cores * NUM_CHUNKS_PER_CORE))
   chunk_size = min(MAX_CHUNK_SIZE, max(MIN_CHUNK_SIZE, total_size / intended_num_chunks))
   chunks = shared.chunkify(funcs, chunk_size)
 
   chunks = [chunk for chunk in chunks if len(chunk)]
-  if DEBUG and len(chunks): print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
+  if DEBUG and len(chunks):
+    print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
   funcs = None
 
   if len(chunks):
     def write_chunk(chunk, i):
       temp_file = temp_files.get('.jsfunc_%d.js' % i).name
-      f = open(temp_file, 'w')
-      f.write(chunk)
-
-      if not gen_hash_info:
-        f.write('\n')
-        f.write(equivalentfn_hash_info)
-      f.close()
+      with open(temp_file, 'w') as f:
+        f.write(chunk)
+        if not gen_hash_info:
+          f.write('\n')
+          f.write(equivalentfn_hash_info)
       return temp_file
     filenames = [write_chunk(chunks[i], i) for i in range(len(chunks))]
   else:
@@ -244,7 +263,7 @@ def run_on_js(filename, gen_hash_info=False):
 
   old_filenames = filenames[:]
   if len(filenames):
-    commands = [js_engine + [DUPLICATE_FUNCTION_ELIMINATOR, filename, '--gen-hash-info' if gen_hash_info else '--use-hash-info', '--no-minimize-whitespace'] for filename in filenames]
+    commands = [js_engine + [DUPLICATE_FUNCTION_ELIMINATOR, f, '--gen-hash-info' if gen_hash_info else '--use-hash-info', '--no-minimize-whitespace'] for f in filenames]
 
     if DEBUG and commands is not None:
       print([' '.join(command if command is not None else '(null)') for command in commands], file=sys.stderr)
@@ -252,12 +271,14 @@ def run_on_js(filename, gen_hash_info=False):
     cores = min(cores, len(filenames))
     if len(chunks) > 1 and cores >= 2:
       # We can parallelize
-      if DEBUG: print('splitting up js optimization into %d chunks, using %d cores  (total: %.2f MB)' % (len(chunks), cores, total_size/(1024*1024.)), file=sys.stderr)
-      pool = shared.Building.get_multiprocessing_pool()
+      if DEBUG:
+        print('splitting up js optimization into %d chunks, using %d cores  (total: %.2f MB)' % (len(chunks), cores, total_size / (1024 * 1024.)), file=sys.stderr)
+      pool = building.get_multiprocessing_pool()
       filenames = pool.map(run_on_chunk, commands, chunksize=1)
     else:
       # We can't parallize, but still break into chunks to avoid uglify/node memory issues
-      if len(chunks) > 1 and DEBUG: print('splitting up js optimization into %d chunks' % (len(chunks)), file=sys.stderr)
+      if len(chunks) > 1 and DEBUG:
+        print('splitting up js optimization into %d chunks' % (len(chunks)), file=sys.stderr)
       filenames = [run_on_chunk(command) for command in commands]
   else:
     filenames = []
@@ -275,11 +296,12 @@ def run_on_js(filename, gen_hash_info=False):
     json_files = filenames[:]
     filenames = old_filenames[:]
 
-  for filename in filenames: temp_files.note(filename)
+  for filename in filenames:
+    temp_files.note(filename)
 
   filename += '.jo.js'
   f = open(filename, 'w')
-  f.write(pre);
+  f.write(pre)
   pre = None
 
   # sort functions by size, to make diffing easier and to improve aot times
@@ -296,7 +318,7 @@ def run_on_js(filename, gen_hash_info=False):
   funcs = None
 
   f.write('\n')
-  f.write(post);
+  f.write(post)
   # No need to write suffix: if there was one, it is inside post which exists when suffix is there
   f.write('\n')
 
@@ -306,11 +328,12 @@ def run_on_js(filename, gen_hash_info=False):
 
   return filename
 
+
 def save_temp_file(file_to_process):
   if os.environ.get('EMSCRIPTEN_SAVE_TEMP_FILES') and os.environ.get('EMSCRIPTEN_TEMP_FILES_DIR'):
     destinationFile = file_to_process
 
-    temp_dir_name = os.environ.get('TEMP_DIR')
+    temp_dir_name = tempfile.gettempdir()
     destinationFile = destinationFile.replace(temp_dir_name, os.environ.get('EMSCRIPTEN_TEMP_FILES_DIR'))
 
     if not os.path.exists(os.path.dirname(destinationFile)):
@@ -319,6 +342,7 @@ def save_temp_file(file_to_process):
     print("Copying {} to {}".format(file_to_process, destinationFile), file=sys.stderr)
     shutil.copyfile(file_to_process, destinationFile)
 
+
 def get_func_names(javascript_file):
   func_names = []
   start_tok = "// EMSCRIPTEN_START_FUNCS"
@@ -326,17 +350,18 @@ def get_func_names(javascript_file):
   start_off = 0
   end_off = 0
 
-  with open (javascript_file, 'rt') as fin:
+  with open(javascript_file, 'rt') as fin:
     blob = "".join(fin.readlines())
     start_off = blob.find(start_tok) + len(start_tok)
     end_off = blob.find(end_tok)
     asm_chunk = blob[start_off:end_off]
 
-    for match in re.finditer('function (\S+?)\s*\(', asm_chunk):
+    for match in re.finditer(r'function (\S+?)\s*\(', asm_chunk):
       func_names.append(match.groups(1)[0])
 
   return func_names
 
+
 def eliminate_duplicate_funcs(file_name):
   if shared.Settings.ELIMINATE_DUPLICATE_FUNCTIONS_DUMP_EQUIVALENT_FUNCTIONS != 0:
     # Remove previous log file if it exists
@@ -348,7 +373,8 @@ def eliminate_duplicate_funcs(file_name):
     old_funcs = get_func_names(file_name)
 
   for pass_num in range(shared.Settings.ELIMINATE_DUPLICATE_FUNCTIONS_PASSES):
-    if DEBUG: print("[PASS {}]: eliminating duplicate functions in: {}.".format(pass_num, file_name), file=sys.stderr)
+    if DEBUG:
+      print("[PASS {}]: eliminating duplicate functions in: {}.".format(pass_num, file_name), file=sys.stderr)
 
     # Generate the JSON for the equivalent hash first
     processed_file = run_on_js(filename=file_name, gen_hash_info=True)
@@ -370,16 +396,18 @@ def eliminate_duplicate_funcs(file_name):
     print("Writing eliminated functions to file: {}".format(eliminated_funcs_file), file=sys.stderr)
 
     with open(eliminated_funcs_file, 'w') as fout:
-      eliminated_functions = list(set(old_funcs)-set(new_funcs))
+      eliminated_functions = list(set(old_funcs) - set(new_funcs))
       eliminated_functions.sort()
       for eliminated_function in eliminated_functions:
         fout.write('{}\n'.format(eliminated_function))
 
+
 def run(filename, js_engine=shared.NODE_JS):
   js_engine = shared.listify(js_engine)
 
   return temp_files.run_and_clean(lambda: eliminate_duplicate_funcs(filename))
 
-if __name__ == '__main__':
-  out = run(sys.argv[1], sys.argv[2:])
 
+if __name__ == '__main__':
+  run(sys.argv[1], sys.argv[2:])
+  sys.exit(0)
